@@ -629,7 +629,7 @@ void AgendaItem::addAttendee(const QString &newAttendee)
         mIncidence->addAttendee(KCalendarCore::Attendee(name, email));
         KMessageBox::information(this,
                                  i18n("Attendee \"%1\" added to the calendar item \"%2\"", KEmailAddress::normalizedAddress(name, email, QString()), text()),
-                                 i18n("Attendee added"),
+                                 i18nc("@title:window", "Attendee added"),
                                  QStringLiteral("AttendeeDroppedAdded"));
     }
 }
@@ -1037,11 +1037,7 @@ void AgendaItem::paintEvent(QPaintEvent *ev)
     p.setBackground(QBrush(bgColor));
     p.setPen(textColor);
     QString ws = ww.wrappedString();
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    if (ws.leftRef(ws.length() - 1).indexOf(QLatin1Char('\n')) >= 0) {
-#else
     if (QStringView(ws).left(ws.length() - 1).indexOf(QLatin1Char('\n')) >= 0) {
-#endif
         ww.drawText(&p, eventX, y, Qt::AlignLeft | KWordWrap::FadeOut);
     } else {
         ww.drawText(&p, eventX + (txtWidth - ww.boundingRect().width() - 2 * margin) / 2, y, Qt::AlignHCenter | KWordWrap::FadeOut);
@@ -1189,3 +1185,5 @@ bool AgendaItem::event(QEvent *event)
     }
     return QWidget::event(event);
 }
+
+#include "moc_agendaitem.cpp"
