@@ -32,14 +32,14 @@ public:
     explicit JournalView(QWidget *parent = nullptr);
     ~JournalView() override;
 
-    Q_REQUIRED_RESULT int currentDateCount() const override;
-    Q_REQUIRED_RESULT Akonadi::Item::List selectedIncidences() const override;
-    Q_REQUIRED_RESULT KCalendarCore::DateList selectedIncidenceDates() const override
+    [[nodiscard]] int currentDateCount() const override;
+    [[nodiscard]] Akonadi::Item::List selectedIncidences() const override;
+    [[nodiscard]] KCalendarCore::DateList selectedIncidenceDates() const override
     {
         return {};
     }
 
-    void appendJournal(const Akonadi::Item &journal, QDate dt);
+    void appendJournal(const Akonadi::Item &journal, const Akonadi::CollectionCalendar::Ptr &calendar, QDate dt);
 
     /** documentation in baseview.h */
     void getHighlightMode(bool &highlightEvents, bool &highlightTodos, bool &highlightJournals);
@@ -72,6 +72,8 @@ protected:
     void clearEntries();
 
 private:
+    Akonadi::CollectionCalendar::Ptr findCalendar(const KCalendarCore::Journal::Ptr &journal) const;
+
     QScrollArea *mSA = nullptr;
     QWidget *mCurrentWidget = nullptr;
     QMap<QDate, EventViews::JournalDateView *> mEntries;
@@ -79,4 +81,3 @@ private:
     //    DateList mSelectedDates;  // List of dates to be displayed
 };
 }
-
