@@ -14,11 +14,11 @@
 #include "configdialoginterface.h"
 #include "prefs.h"
 
-#include <AkonadiCore/EntityTreeModel>
-#include <AkonadiWidgets/ETMViewStateSaver>
+#include <Akonadi/CalendarUtils>
+#include <Akonadi/ETMViewStateSaver>
+#include <Akonadi/EntityTreeModel>
 
 #include <CalendarSupport/CollectionSelection>
-#include <CalendarSupport/Utils>
 
 #include <KCheckableProxyModel>
 #include <KLocalizedString>
@@ -72,48 +72,51 @@ static void printObject( QObject *o, int level = 0 )
 }
 */
 
+class DefaultCalendarFactory : public MultiAgendaView::CalendarFactory
+{
+public:
+    using Ptr = QSharedPointer<DefaultCalendarFactory>;
+
+    explicit DefaultCalendarFactory(MultiAgendaView *view)
+        : mView(view)
+    {
+    }
+
+    Akonadi::CollectionCalendar::Ptr calendarForCollection(const Akonadi::Collection &collection) override
+    {
+        return Akonadi::CollectionCalendar::Ptr::create(mView->entityTreeModel(), collection);
+    }
+
+private:
+    MultiAgendaView *mView;
+};
+
 static QString generateColumnLabel(int c)
 {
     return i18n("Agenda %1", c + 1);
 }
 
-class Q_DECL_HIDDEN MultiAgendaView::Private
+class EventViews::MultiAgendaViewPrivate
 {
-private:
-    class ElidedLabel : public QFrame
-    {
-    public:
-        ElidedLabel(const QString &text)
-            : mText(text)
-        {
-        }
-
-        QSize minimumSizeHint() const override;
-
-    protected:
-        void paintEvent(QPaintEvent *event) override;
-
-    private:
-        const QString mText;
-    };
-
 public:
-    Private(MultiAgendaView *qq)
+    explicit MultiAgendaViewPrivate(const MultiAgendaView::CalendarFactory::Ptr &factory, MultiAgendaView *qq)
         : q(qq)
+        , mCalendarFactory(factory)
     {
     }
 
-    ~Private()
+    ~MultiAgendaViewPrivate()
     {
         qDeleteAll(mSelectionSavers);
     }
 
-    void addView(const Akonadi::Collection &collection);
+    void addView(const Akonadi::CollectionCalendar::Ptr &calendar);
     void addView(KCheckableProxyModel *selectionProxy, const QString &title);
-    AgendaView *createView(const QString &title);
+    AgendaView *createView(const QString &calendar);
     void deleteViews();
     void setupViews();
     void resizeScrollView(QSize size);
+    void setActiveAgenda(AgendaView *view);
 
     MultiAgendaView *const q;
     QList<AgendaView *> mAgendaViews;
@@ -130,113 +133,146 @@ public:
     bool mUpdateOnShow = true;
     bool mPendingChanges = true;
     bool mCustomColumnSetupUsed = false;
-    QVector<KCheckableProxyModel *> mCollectionSelectionModels;
+    QList<KCheckableProxyModel *> mCollectionSelectionModels;
     QStringList mCustomColumnTitles;
     int mCustomNumberOfColumns = 2;
     QLabel *mLabel = nullptr;
     QWidget *mRightDummyWidget = nullptr;
     QHash<QString, KViewStateMaintainer<ETMViewStateSaver> *> mSelectionSavers;
+    QMetaObject::Connection m_selectionChangeConn;
+    MultiAgendaView::CalendarFactory::Ptr mCalendarFactory;
 };
 
 MultiAgendaView::MultiAgendaView(QWidget *parent)
+    : MultiAgendaView(DefaultCalendarFactory::Ptr::create(this), parent)
+{
+}
+
+MultiAgendaView::MultiAgendaView(const CalendarFactory::Ptr &factory, QWidget *parent)
     : EventView(parent)
-    , d(new Private(this))
+    , d(new MultiAgendaViewPrivate(factory, this))
 {
     auto topLevelLayout = new QHBoxLayout(this);
     topLevelLayout->setSpacing(0);
     topLevelLayout->setContentsMargins(0, 0, 0, 0);
 
-    QFontMetrics fm(font());
-    int topLabelHeight = 2 * fm.height() + fm.lineSpacing();
+    // agendaheader is a VBox layout with default spacing containing two labels,
+    // so the height is 2 * default font height + 2 * default vertical layout spacing
+    // (that's vertical spacing between the labels and spacing between the header and the
+    // top of the agenda grid)
+    const auto spacing = style()->pixelMetric(QStyle::PM_LayoutVerticalSpacing, nullptr, this);
+    const int agendeHeaderHeight = 2 * QFontMetrics(font()).height() + 2 * spacing;
 
-    auto topSideBox = new QWidget(this);
-    auto topSideBoxVBoxLayout = new QVBoxLayout(topSideBox);
-    topSideBoxVBoxLayout->setContentsMargins(0, 0, 0, 0);
+    // Left sidebox
+    {
+        auto sideBox = new QWidget(this);
+        auto sideBoxLayout = new QVBoxLayout(sideBox);
+        sideBoxLayout->setSpacing(0);
+        sideBoxLayout->setContentsMargins(0, agendeHeaderHeight, 0, 0);
 
-    auto topSideSpacer = new QWidget(topSideBox);
-    topSideBoxVBoxLayout->addWidget(topSideSpacer);
-    topSideSpacer->setFixedHeight(topLabelHeight);
+        // Splitter for full-day and regular agenda views
+        d->mLeftSplitter = new QSplitter(Qt::Vertical, sideBox);
+        sideBoxLayout->addWidget(d->mLeftSplitter, 1);
 
-    d->mLeftSplitter = new QSplitter(Qt::Vertical, topSideBox);
-    topSideBoxVBoxLayout->addWidget(d->mLeftSplitter);
+        // Label for all-day view
+        d->mLabel = new QLabel(i18n("All Day"), d->mLeftSplitter);
+        d->mLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
+        d->mLabel->setWordWrap(true);
 
-    d->mLabel = new QLabel(i18n("All Day"), d->mLeftSplitter);
-    d->mLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
-    d->mLabel->setWordWrap(true);
+        auto timeLabelsBox = new QWidget(d->mLeftSplitter);
+        auto timeLabelsBoxLayout = new QVBoxLayout(timeLabelsBox);
+        timeLabelsBoxLayout->setSpacing(0);
+        timeLabelsBoxLayout->setContentsMargins(0, 0, 0, 0);
 
-    auto sideBox = new QWidget(d->mLeftSplitter);
-    auto sideBoxVBoxLayout = new QVBoxLayout(sideBox);
-    sideBoxVBoxLayout->setContentsMargins(0, 0, 0, 0);
+        d->mTimeLabelsZone = new TimeLabelsZone(timeLabelsBox, PrefsPtr(new Prefs()));
+        timeLabelsBoxLayout->addWidget(d->mTimeLabelsZone);
 
-    // compensate for the frame the agenda views but not the timelabels have
-    auto timeLabelTopAlignmentSpacer = new QWidget(sideBox);
-    sideBoxVBoxLayout->addWidget(timeLabelTopAlignmentSpacer);
+        // Compensate for horizontal scrollbars, if needed
+        d->mLeftBottomSpacer = new QWidget(timeLabelsBox);
+        timeLabelsBoxLayout->addWidget(d->mLeftBottomSpacer);
 
-    d->mTimeLabelsZone = new TimeLabelsZone(sideBox, PrefsPtr(new Prefs()));
+        topLevelLayout->addWidget(sideBox);
+    }
 
-    auto timeLabelBotAlignmentSpacer = new QWidget(sideBox);
-    sideBoxVBoxLayout->addWidget(timeLabelBotAlignmentSpacer);
+    // Central area
+    {
+        d->mScrollArea = new QScrollArea(this);
+        d->mScrollArea->setWidgetResizable(true);
 
-    d->mLeftBottomSpacer = new QWidget(topSideBox);
-    topSideBoxVBoxLayout->addWidget(d->mLeftBottomSpacer);
+        d->mScrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+        d->mScrollArea->setFrameShape(QFrame::NoFrame);
 
-    topLevelLayout->addWidget(topSideBox);
+        d->mTopBox = new QWidget(d->mScrollArea->viewport());
+        auto mTopBoxHBoxLayout = new QHBoxLayout(d->mTopBox);
+        mTopBoxHBoxLayout->setContentsMargins(0, 0, 0, 0);
+        d->mScrollArea->setWidget(d->mTopBox);
 
-    d->mScrollArea = new QScrollArea(this);
-    d->mScrollArea->setWidgetResizable(true);
+        topLevelLayout->addWidget(d->mScrollArea, 100);
+    }
 
-    d->mScrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    // Right side box (scrollbar)
+    {
+        auto sideBox = new QWidget(this);
+        auto sideBoxLayout = new QVBoxLayout(sideBox);
+        sideBoxLayout->setSpacing(0);
+        sideBoxLayout->setContentsMargins(0, agendeHeaderHeight, 0, 0);
 
-    // BUG: timelabels aren't aligned with the agenda's grid, 2 or 3 pixels of offset.
-    // asymetric since the timelabels
-    timeLabelTopAlignmentSpacer->setFixedHeight(d->mScrollArea->frameWidth() - 1);
-    // have 25 horizontal lines
-    timeLabelBotAlignmentSpacer->setFixedHeight(d->mScrollArea->frameWidth() - 2);
+        d->mRightSplitter = new QSplitter(Qt::Vertical, sideBox);
+        sideBoxLayout->addWidget(d->mRightSplitter);
 
-    d->mScrollArea->setFrameShape(QFrame::NoFrame);
-    topLevelLayout->addWidget(d->mScrollArea, 100);
-    d->mTopBox = new QWidget(d->mScrollArea->viewport());
-    auto mTopBoxHBoxLayout = new QHBoxLayout(d->mTopBox);
-    mTopBoxHBoxLayout->setContentsMargins(0, 0, 0, 0);
-    d->mScrollArea->setWidget(d->mTopBox);
+        // Empty widget, equivalent to mLabel in the left box
+        d->mRightDummyWidget = new QWidget(d->mRightSplitter);
 
-    topSideBox = new QWidget(this);
-    topSideBoxVBoxLayout = new QVBoxLayout(topSideBox);
-    topSideBoxVBoxLayout->setContentsMargins(0, 0, 0, 0);
+        d->mScrollBar = new QScrollBar(Qt::Vertical, d->mRightSplitter);
 
-    topSideSpacer = new QWidget(topSideBox);
-    topSideBoxVBoxLayout->addWidget(topSideSpacer);
-    topSideSpacer->setFixedHeight(topLabelHeight);
+        // Compensate for horizontal scrollbar, if needed
+        d->mRightBottomSpacer = new QWidget(sideBox);
+        sideBoxLayout->addWidget(d->mRightBottomSpacer);
 
-    d->mRightSplitter = new QSplitter(Qt::Vertical, topSideBox);
-    topSideBoxVBoxLayout->addWidget(d->mRightSplitter);
+        topLevelLayout->addWidget(sideBox);
+    }
+
+    // BUG: compensate for agenda view's frames to make sure time labels are aligned
+    d->mTimeLabelsZone->setContentsMargins(0, d->mScrollArea->frameWidth(), 0, d->mScrollArea->frameWidth());
 
     connect(d->mLeftSplitter, &QSplitter::splitterMoved, this, &MultiAgendaView::resizeSplitters);
     connect(d->mRightSplitter, &QSplitter::splitterMoved, this, &MultiAgendaView::resizeSplitters);
-
-    d->mRightDummyWidget = new QWidget(d->mRightSplitter);
-
-    d->mScrollBar = new QScrollBar(Qt::Vertical, d->mRightSplitter);
-
-    d->mRightBottomSpacer = new QWidget(topSideBox);
-    topSideBoxVBoxLayout->addWidget(d->mRightBottomSpacer);
-    topLevelLayout->addWidget(topSideBox);
 }
 
-void MultiAgendaView::setCalendar(const Akonadi::ETMCalendar::Ptr &calendar)
+void MultiAgendaView::addCalendar(const Akonadi::CollectionCalendar::Ptr &calendar)
 {
-    EventView::setCalendar(calendar);
-    for (KCheckableProxyModel *proxy : std::as_const(d->mCollectionSelectionModels)) {
-        proxy->setSourceModel(calendar->entityTreeModel());
-    }
-
-    disconnect(nullptr, SIGNAL(selectionChanged(Akonadi::Collection::List, Akonadi::Collection::List)), this, SLOT(forceRecreateViews()));
-
-    connect(collectionSelection(), &CalendarSupport::CollectionSelection::selectionChanged, this, &MultiAgendaView::forceRecreateViews);
-
+    EventView::addCalendar(calendar);
+    d->mPendingChanges = true;
     recreateViews();
 }
 
+void MultiAgendaView::removeCalendar(const Akonadi::CollectionCalendar::Ptr &calendar)
+{
+    EventView::removeCalendar(calendar);
+    d->mPendingChanges = true;
+    recreateViews();
+}
+
+void MultiAgendaView::setModel(QAbstractItemModel *model)
+{
+    EventView::setModel(model);
+    // Workaround: when we create the multiagendaview with custom columns too early
+    // during start, when Collections in ETM are not fully loaded yet, then
+    // the KCheckableProxyModels are restored from config with incomplete selections.
+    // But when the Collections are finally loaded into ETM, there's nothing to update
+    // the selections, so we end up with some calendars not displayed in the individual
+    // AgendaViews. Thus, we force-recreate everything once collection tree is fetched.
+    connect(
+        entityTreeModel(),
+        &Akonadi::EntityTreeModel::collectionTreeFetched,
+        this,
+        [this]() {
+            d->mPendingChanges = true;
+            recreateViews();
+        },
+        Qt::QueuedConnection);
+}
+
 void MultiAgendaView::recreateViews()
 {
     if (!d->mPendingChanges) {
@@ -252,13 +288,11 @@ void MultiAgendaView::recreateViews()
             d->addView(d->mCollectionSelectionModels[i], d->mCustomColumnTitles[i]);
         }
     } else {
-        const auto lst = collectionSelection()->selectedCollections();
-        for (const Akonadi::Collection &i : lst) {
-            if (i.contentMimeTypes().contains(KCalendarCore::Event::eventMimeType())) {
-                d->addView(i);
-            }
+        for (const auto &calendar : calendars()) {
+            d->addView(calendar);
         }
     }
+
     // no resources activated, so stop here to avoid crashing somewhere down the line
     // TODO: show a nice message instead
     if (d->mAgendaViews.isEmpty()) {
@@ -273,6 +307,11 @@ void MultiAgendaView::recreateViews()
     connect(timeLabel->verticalScrollBar(), &QAbstractSlider::valueChanged, d->mScrollBar, &QAbstractSlider::setValue);
     connect(d->mScrollBar, &QAbstractSlider::valueChanged, timeLabel->verticalScrollBar(), &QAbstractSlider::setValue);
 
+    // On initial view, sync our splitter sizes with the agenda
+    if (d->mAgendaViews.size() == 1) {
+        d->mLeftSplitter->setSizes(d->mAgendaViews[0]->splitter()->sizes());
+        d->mRightSplitter->setSizes(d->mAgendaViews[0]->splitter()->sizes());
+    }
     resizeSplitters();
     QTimer::singleShot(0, this, &MultiAgendaView::setupScrollBar);
 
@@ -285,7 +324,7 @@ void MultiAgendaView::forceRecreateViews()
     recreateViews();
 }
 
-void MultiAgendaView::Private::deleteViews()
+void MultiAgendaViewPrivate::deleteViews()
 {
     for (AgendaView *const i : std::as_const(mAgendaViews)) {
         KCheckableProxyModel *proxy = i->takeCustomCollectionSelectionProxyModel();
@@ -301,40 +340,46 @@ void MultiAgendaView::Private::deleteViews()
     mAgendaWidgets.clear();
 }
 
-void MultiAgendaView::Private::setupViews()
+void MultiAgendaViewPrivate::setupViews()
 {
-    for (AgendaView *agenda : std::as_const(mAgendaViews)) {
-        q->connect(agenda, SIGNAL(newEventSignal()), q, SIGNAL(newEventSignal()));
-        q->connect(agenda, SIGNAL(newEventSignal(QDate)), q, SIGNAL(newEventSignal(QDate)));
-        q->connect(agenda, SIGNAL(newEventSignal(QDateTime)), q, SIGNAL(newEventSignal(QDateTime)));
-        q->connect(agenda, SIGNAL(newEventSignal(QDateTime, QDateTime)), q, SIGNAL(newEventSignal(QDateTime, QDateTime)));
+    for (AgendaView *agendaView : std::as_const(mAgendaViews)) {
+        q->connect(agendaView, qOverload<>(&EventView::newEventSignal), q, qOverload<>(&EventView::newEventSignal));
+        q->connect(agendaView, qOverload<const QDate &>(&EventView::newEventSignal), q, qOverload<const QDate &>(&EventView::newEventSignal));
+        q->connect(agendaView, qOverload<const QDateTime &>(&EventView::newEventSignal), q, qOverload<const QDateTime &>(&EventView::newEventSignal));
+        q->connect(agendaView,
+                   qOverload<const QDateTime &, const QDateTime &>(&EventView::newEventSignal),
+                   q,
+                   qOverload<const QDateTime &>(&EventView::newEventSignal));
 
-        q->connect(agenda, &EventView::editIncidenceSignal, q, &EventView::editIncidenceSignal);
-        q->connect(agenda, &EventView::showIncidenceSignal, q, &EventView::showIncidenceSignal);
-        q->connect(agenda, &EventView::deleteIncidenceSignal, q, &EventView::deleteIncidenceSignal);
+        q->connect(agendaView, &EventView::editIncidenceSignal, q, &EventView::editIncidenceSignal);
+        q->connect(agendaView, &EventView::showIncidenceSignal, q, &EventView::showIncidenceSignal);
+        q->connect(agendaView, &EventView::deleteIncidenceSignal, q, &EventView::deleteIncidenceSignal);
 
-        q->connect(agenda, &EventView::incidenceSelected, q, &EventView::incidenceSelected);
+        q->connect(agendaView, &EventView::incidenceSelected, q, &EventView::incidenceSelected);
 
-        q->connect(agenda, &EventView::cutIncidenceSignal, q, &EventView::cutIncidenceSignal);
-        q->connect(agenda, &EventView::copyIncidenceSignal, q, &EventView::copyIncidenceSignal);
-        q->connect(agenda, &EventView::pasteIncidenceSignal, q, &EventView::pasteIncidenceSignal);
-        q->connect(agenda, &EventView::toggleAlarmSignal, q, &EventView::toggleAlarmSignal);
-        q->connect(agenda, &EventView::dissociateOccurrencesSignal, q, &EventView::dissociateOccurrencesSignal);
+        q->connect(agendaView, &EventView::cutIncidenceSignal, q, &EventView::cutIncidenceSignal);
+        q->connect(agendaView, &EventView::copyIncidenceSignal, q, &EventView::copyIncidenceSignal);
+        q->connect(agendaView, &EventView::pasteIncidenceSignal, q, &EventView::pasteIncidenceSignal);
+        q->connect(agendaView, &EventView::toggleAlarmSignal, q, &EventView::toggleAlarmSignal);
+        q->connect(agendaView, &EventView::dissociateOccurrencesSignal, q, &EventView::dissociateOccurrencesSignal);
 
-        q->connect(agenda, &EventView::newTodoSignal, q, &EventView::newTodoSignal);
+        q->connect(agendaView, &EventView::newTodoSignal, q, &EventView::newTodoSignal);
 
-        q->connect(agenda, &EventView::incidenceSelected, q, &MultiAgendaView::slotSelectionChanged);
+        q->connect(agendaView, &EventView::incidenceSelected, q, &MultiAgendaView::slotSelectionChanged);
 
-        q->connect(agenda, &AgendaView::timeSpanSelectionChanged, q, &MultiAgendaView::slotClearTimeSpanSelection);
+        q->connect(agendaView, &AgendaView::timeSpanSelectionChanged, q, &MultiAgendaView::slotClearTimeSpanSelection);
 
-        q->disconnect(agenda->agenda(), SIGNAL(zoomView(int, QPoint, Qt::Orientation)), agenda, nullptr);
-        q->connect(agenda->agenda(), &Agenda::zoomView, q, &MultiAgendaView::zoomView);
+        q->disconnect(agendaView->agenda(), &Agenda::zoomView, agendaView, nullptr);
+        q->connect(agendaView->agenda(), &Agenda::zoomView, q, &MultiAgendaView::zoomView);
     }
 
     AgendaView *lastView = mAgendaViews.last();
-    for (AgendaView *agenda : std::as_const(mAgendaViews)) {
-        if (agenda != lastView) {
-            connect(agenda->agenda()->verticalScrollBar(), &QAbstractSlider::valueChanged, lastView->agenda()->verticalScrollBar(), &QAbstractSlider::setValue);
+    for (AgendaView *agendaView : std::as_const(mAgendaViews)) {
+        if (agendaView != lastView) {
+            q->connect(agendaView->agenda()->verticalScrollBar(),
+                       &QAbstractSlider::valueChanged,
+                       lastView->agenda()->verticalScrollBar(),
+                       &QAbstractSlider::setValue);
         }
     }
 
@@ -343,10 +388,7 @@ void MultiAgendaView::Private::setupViews()
     }
 }
 
-MultiAgendaView::~MultiAgendaView()
-{
-    delete d;
-}
+MultiAgendaView::~MultiAgendaView() = default;
 
 Akonadi::Item::List MultiAgendaView::selectedIncidences() const
 {
@@ -427,28 +469,48 @@ bool MultiAgendaView::eventDurationHint(QDateTime &startDt, QDateTime &endDt, bo
     return false;
 }
 
+// Invoked when user selects a cell or a span of cells in agendaview
 void MultiAgendaView::slotClearTimeSpanSelection()
 {
     for (AgendaView *agenda : std::as_const(d->mAgendaViews)) {
         if (agenda != sender()) {
             agenda->clearTimeSpanSelection();
-        } else {
-            setCollectionId(agenda->collectionId());
+        } else if (!d->mCustomColumnSetupUsed) {
+            d->setActiveAgenda(agenda);
         }
     }
 }
 
-AgendaView *MultiAgendaView::Private::createView(const QString &title)
+void MultiAgendaViewPrivate::setActiveAgenda(AgendaView *view)
+{
+    // Only makes sense in the one-agenda-per-calendar set up
+    if (mCustomColumnSetupUsed) {
+        return;
+    }
+
+    if (!view) {
+        return;
+    }
+
+    auto calendars = view->calendars();
+    if (calendars.empty()) {
+        return;
+    }
+    Q_ASSERT(calendars.size() == 1);
+
+    Q_EMIT q->activeCalendarChanged(calendars.at(0));
+}
+
+AgendaView *MultiAgendaViewPrivate::createView(const QString &title)
 {
     auto box = new QWidget(mTopBox);
     mTopBox->layout()->addWidget(box);
     auto layout = new QVBoxLayout(box);
     layout->setContentsMargins(0, 0, 0, 0);
-    layout->addWidget(new ElidedLabel(title));
     auto av = new AgendaView(q->preferences(), q->startDateTime().date(), q->endDateTime().date(), true, true, q);
     layout->addWidget(av);
-    av->setCalendar(q->calendar());
     av->setIncidenceChanger(q->changer());
+    av->setTitle(title);
     av->agenda()->scrollArea()->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     mAgendaViews.append(av);
     mAgendaWidgets.append(box);
@@ -458,6 +520,8 @@ AgendaView *MultiAgendaView::Private::createView(const QString &title)
     q->connect(mScrollBar, &QAbstractSlider::valueChanged, av->agenda()->verticalScrollBar(), &QAbstractSlider::setValue);
 
     q->connect(av->splitter(), &QSplitter::splitterMoved, q, &MultiAgendaView::resizeSplitters);
+    // The change in all-day and regular agenda height ratio affects scrollbars as well
+    q->connect(av->splitter(), &QSplitter::splitterMoved, q, &MultiAgendaView::setupScrollBar);
     q->connect(av, &AgendaView::showIncidencePopupSignal, q, &MultiAgendaView::showIncidencePopupSignal);
 
     q->connect(av, &AgendaView::showNewEventPopupSignal, q, &MultiAgendaView::showNewEventPopupSignal);
@@ -472,25 +536,58 @@ AgendaView *MultiAgendaView::Private::createView(const QString &title)
     return av;
 }
 
-void MultiAgendaView::Private::addView(const Akonadi::Collection &collection)
+void MultiAgendaViewPrivate::addView(const Akonadi::CollectionCalendar::Ptr &calendar)
 {
-    AgendaView *av = createView(CalendarSupport::displayName(q->calendar().data(), collection));
-    av->setCollectionId(collection.id());
+    const auto title = Akonadi::CalendarUtils::displayName(calendar->model(), calendar->collection());
+    auto *view = createView(title);
+    view->addCalendar(calendar);
 }
 
-void MultiAgendaView::Private::addView(KCheckableProxyModel *sm, const QString &title)
+static void updateViewFromSelection(AgendaView *view,
+                                    const QItemSelection &selected,
+                                    const QItemSelection &deselected,
+                                    const MultiAgendaView::CalendarFactory::Ptr &factory)
 {
-    AgendaView *av = createView(title);
-    av->setCustomCollectionSelectionProxyModel(sm);
+    for (const auto index : selected.indexes()) {
+        if (const auto col = index.data(Akonadi::EntityTreeModel::CollectionRole).value<Akonadi::Collection>(); col.isValid()) {
+            const auto calendar = factory->calendarForCollection(col);
+            view->addCalendar(calendar);
+        }
+    }
+    for (const auto index : deselected.indexes()) {
+        if (const auto col = index.data(Akonadi::EntityTreeModel::CollectionRole).value<Akonadi::Collection>(); col.isValid()) {
+            if (const auto calendar = view->calendarForCollection(col); calendar) {
+                view->removeCalendar(calendar);
+            }
+        }
+    }
+}
+
+void MultiAgendaViewPrivate::addView(KCheckableProxyModel *sm, const QString &title)
+{
+    auto *view = createView(title);
+    // During launch the underlying ETM doesn't have the entire Collection tree populated,
+    // so the "sm" contains an icomplete selection - we must listen for changes and upated
+    // the view later on
+    QObject::connect(sm->selectionModel(),
+                     &QItemSelectionModel::selectionChanged,
+                     view,
+                     [this, view](const QItemSelection &selected, const QItemSelection &deselected) {
+                         updateViewFromSelection(view, selected, deselected, mCalendarFactory);
+                     });
+
+    // Initial update
+    updateViewFromSelection(view, sm->selectionModel()->selection(), QItemSelection{}, mCalendarFactory);
 }
 
 void MultiAgendaView::resizeEvent(QResizeEvent *ev)
 {
     d->resizeScrollView(ev->size());
     EventView::resizeEvent(ev);
+    setupScrollBar();
 }
 
-void MultiAgendaView::Private::resizeScrollView(QSize size)
+void MultiAgendaViewPrivate::resizeScrollView(QSize size)
 {
     const int widgetWidth = size.width() - mTimeLabelsZone->width() - mScrollBar->width();
 
@@ -630,11 +727,13 @@ bool MultiAgendaView::hasConfigurationDialog() const
 
 void MultiAgendaView::doRestoreConfig(const KConfigGroup &configGroup)
 {
+    /*
     if (!calendar()) {
         qCCritical(CALENDARVIEW_LOG) << "Calendar is not set.";
         Q_ASSERT(false);
         return;
     }
+    */
 
     d->mCustomColumnSetupUsed = configGroup.readEntry("UseCustomColumnSetup", false);
     d->mCustomNumberOfColumns = configGroup.readEntry("CustomNumberOfColumns", 2);
@@ -647,7 +746,7 @@ void MultiAgendaView::doRestoreConfig(const KConfigGroup &configGroup)
         }
     }
 
-    QVector<KCheckableProxyModel *> oldModels = d->mCollectionSelectionModels;
+    QList<KCheckableProxyModel *> oldModels = d->mCollectionSelectionModels;
     d->mCollectionSelectionModels.clear();
 
     if (d->mCustomColumnSetupUsed) {
@@ -656,12 +755,11 @@ void MultiAgendaView::doRestoreConfig(const KConfigGroup &configGroup)
             // Sort the calanders by name
             auto sortProxy = new QSortFilterProxyModel(this);
             sortProxy->setDynamicSortFilter(true);
-
-            sortProxy->setSourceModel(calendar()->entityTreeModel());
+            sortProxy->setSourceModel(model());
 
             // Only show the first column
             auto columnFilterProxy = new KRearrangeColumnsProxyModel(this);
-            columnFilterProxy->setSourceColumns(QVector<int>() << Akonadi::ETMCalendar::CollectionTitle);
+            columnFilterProxy->setSourceColumns(QList<int>() << 0);
             columnFilterProxy->setSourceModel(sortProxy);
 
             // Keep track of selection.
@@ -718,7 +816,7 @@ void MultiAgendaView::customCollectionsChanged(ConfigDialogInterface *dlg)
 
     d->mCustomColumnSetupUsed = dlg->useCustomColumns();
     d->mCustomNumberOfColumns = dlg->numberOfColumns();
-    QVector<KCheckableProxyModel *> newModels;
+    QList<KCheckableProxyModel *> newModels;
     newModels.resize(d->mCustomNumberOfColumns);
     d->mCustomColumnTitles.clear();
     d->mCustomColumnTitles.reserve(d->mCustomNumberOfColumns);
@@ -741,7 +839,7 @@ int MultiAgendaView::customNumberOfColumns() const
     return d->mCustomNumberOfColumns;
 }
 
-QVector<KCheckableProxyModel *> MultiAgendaView::collectionSelectionModels() const
+QList<KCheckableProxyModel *> MultiAgendaView::collectionSelectionModels() const
 {
     return d->mCollectionSelectionModels;
 }
@@ -751,17 +849,4 @@ QStringList MultiAgendaView::customColumnTitles() const
     return d->mCustomColumnTitles;
 }
 
-void MultiAgendaView::Private::ElidedLabel::paintEvent(QPaintEvent *event)
-{
-    QFrame::paintEvent(event);
-    QPainter p(this);
-    QRect r = contentsRect();
-    const QString elidedText = fontMetrics().elidedText(mText, Qt::ElideMiddle, r.width());
-    p.drawText(r, Qt::AlignHCenter | Qt::AlignVCenter, elidedText);
-}
-
-QSize MultiAgendaView::Private::ElidedLabel::minimumSizeHint() const
-{
-    const QFontMetrics &fm = fontMetrics();
-    return QSize(fm.boundingRect(QStringLiteral("...")).width(), fm.height());
-}
+#include "moc_multiagendaview.cpp"
