@@ -22,6 +22,7 @@
 
 #include <Akonadi/CalendarUtils>
 #include <Akonadi/EntityMimeTypeFilterModel>
+#include <Akonadi/EntityTreeModel>
 #include <Akonadi/TagFetchJob>
 
 #include <Akonadi/ETMViewStateSaver>
@@ -34,6 +35,7 @@
 
 #include <KConfig>
 #include <KDatePickerPopup>
+#include <KDescendantsProxyModel>
 #include <KJob>
 #include <KMessageBox>
 
@@ -41,7 +43,9 @@
 #include <QHeaderView>
 #include <QIcon>
 #include <QMenu>
+#include <QSortFilterProxyModel>
 #include <QToolButton>
+
 #include <chrono>
 
 using namespace std::chrono_literals;
@@ -53,6 +57,51 @@ using namespace KCalendarCore;
 
 namespace EventViews
 {
+
+class CalendarFilterModel : public QSortFilterProxyModel
+{
+    Q_OBJECT
+public:
+    explicit CalendarFilterModel(QObject *parent = nullptr)
+        : QSortFilterProxyModel(parent)
+    {
+        mDescendantsProxy.setDisplayAncestorData(false);
+        QSortFilterProxyModel::setSourceModel(&mDescendantsProxy);
+    }
+
+    void setSourceModel(QAbstractItemModel *model) override
+    {
+        mDescendantsProxy.setSourceModel(model);
+    }
+
+    bool filterAcceptsRow(int source_row, const QModelIndex &source_parent) const override
+    {
+        const auto source_index = sourceModel()->index(source_row, 0, source_parent);
+        const auto item = sourceModel()->data(source_index, Akonadi::EntityTreeModel::ItemRole).value<Akonadi::Item>();
+
+        if (!item.isValid()) {
+            return false;
+        }
+        return mEnabledCalendars.contains(item.parentCollection().id());
+    }
+
+    void addCalendar(const Akonadi::CollectionCalendar::Ptr &calendar)
+    {
+        mEnabledCalendars.insert(calendar->collection().id());
+        invalidateFilter();
+    }
+
+    void removeCalendar(const Akonadi::CollectionCalendar::Ptr &calendar)
+    {
+        mEnabledCalendars.remove(calendar->collection().id());
+        invalidateFilter();
+    }
+
+private:
+    KDescendantsProxyModel mDescendantsProxy;
+    QSet<Akonadi::Collection::Id> mEnabledCalendars;
+};
+
 // We share this struct between all views, for performance and memory purposes
 class ModelStack
 {
@@ -61,7 +110,6 @@ public:
         : todoModel(new Akonadi::TodoModel())
         , coloredTodoModel(new ColoredTodoProxyModel(preferences))
         , parent(parent_)
-        , calendar(nullptr)
         , prefs(preferences)
     {
         coloredTodoModel->setSourceModel(todoModel);
@@ -102,7 +150,7 @@ public:
             delete todoFlatModel;
             todoFlatModel = new Akonadi::EntityMimeTypeFilterModel(parent);
             todoFlatModel->addMimeTypeInclusionFilter(todoMimeType);
-            todoFlatModel->setSourceModel(calendar ? calendar->model() : nullptr);
+            todoFlatModel->setSourceModel(model);
             todoModel->setSourceModel(todoFlatModel);
 
             delete todoTreeModel;
@@ -116,7 +164,7 @@ public:
                 view->mView->setDragDropMode(QAbstractItemView::DragDrop);
                 view->setFlatView(flat, /**propagate=*/false); // So other views update their toggle icon
             }
-            todoTreeModel->setSourceModel(calendar ? calendar->model() : nullptr);
+            todoTreeModel->setSourceModel(model);
             todoModel->setSourceModel(todoTreeModel);
             delete todoFlatModel;
             todoFlatModel = nullptr;
@@ -135,12 +183,11 @@ public:
         prefs->writeConfig();
     }
 
-    void setCalendar(const Akonadi::ETMCalendar::Ptr &newCalendar)
+    void setModel(QAbstractItemModel *model)
     {
-        calendar = newCalendar;
-        todoModel->setCalendar(calendar);
+        this->model = model;
         if (todoTreeModel) {
-            todoTreeModel->setSourceModel(calendar ? calendar->model() : nullptr);
+            todoTreeModel->setSourceModel(this->model);
         }
     }
 
@@ -154,7 +201,7 @@ public:
     QList<TodoView *> views;
     QObject *parent = nullptr;
 
-    Akonadi::ETMCalendar::Ptr calendar;
+    QAbstractItemModel *model = nullptr;
     Akonadi::IncidenceTreeModel *todoTreeModel = nullptr;
     Akonadi::EntityMimeTypeFilterModel *todoFlatModel = nullptr;
     EventViews::PrefsPtr prefs;
@@ -166,6 +213,7 @@ static ModelStack *sModels = nullptr;
 
 TodoView::TodoView(const EventViews::PrefsPtr &prefs, bool sidebarView, QWidget *parent)
     : EventView(parent)
+    , mCalendarFilterModel(std::make_unique<CalendarFilterModel>())
     , mQuickSearch(nullptr)
     , mQuickAdd(nullptr)
     , mTreeStateRestorer(nullptr)
@@ -183,11 +231,12 @@ TodoView::TodoView(const EventViews::PrefsPtr &prefs, bool sidebarView, QWidget
         connect(sModels->todoModel, &Akonadi::TodoModel::dropOnSelfRejected, this, []() {
             KMessageBox::information(nullptr,
                                      i18n("Cannot move to-do to itself or a child of itself."),
-                                     i18n("Drop To-do"),
+                                     i18nc("@title:window", "Drop To-do"),
                                      QStringLiteral("NoDropTodoOntoItself"));
         });
     }
     sModels->registerView(this);
+    sModels->setModel(mCalendarFilterModel.get());
 
     mProxyModel = new TodoViewSortFilterProxyModel(preferences(), this);
     mProxyModel->setSourceModel(sModels->coloredTodoModel);
@@ -198,7 +247,7 @@ TodoView::TodoView(const EventViews::PrefsPtr &prefs, bool sidebarView, QWidget
     connect(mProxyModel, &TodoViewSortFilterProxyModel::rowsInserted, this, &TodoView::onRowsInserted);
 
     if (!mSidebarView) {
-        mQuickSearch = new TodoViewQuickSearch(calendar(), this);
+        mQuickSearch = new TodoViewQuickSearch(this);
         mQuickSearch->setVisible(prefs->enableTodoQuickSearch());
         connect(mQuickSearch,
                 &TodoViewQuickSearch::searchTextChanged,
@@ -455,18 +504,26 @@ void TodoView::expandIndex(const QModelIndex &index)
     }
 }
 
-void TodoView::setCalendar(const Akonadi::ETMCalendar::Ptr &calendar)
+void TodoView::setModel(QAbstractItemModel *model)
 {
-    EventView::setCalendar(calendar);
+    EventView::setModel(model);
 
-    if (!mSidebarView) {
-        mQuickSearch->setCalendar(calendar);
-    }
-    mCategoriesDelegate->setCalendar(calendar);
-    sModels->setCalendar(calendar);
+    mCalendarFilterModel->setSourceModel(model);
     restoreViewState();
 }
 
+void TodoView::addCalendar(const Akonadi::CollectionCalendar::Ptr &calendar)
+{
+    EventView::addCalendar(calendar);
+    mCalendarFilterModel->addCalendar(calendar);
+}
+
+void TodoView::removeCalendar(const Akonadi::CollectionCalendar::Ptr &calendar)
+{
+    mCalendarFilterModel->removeCalendar(calendar);
+    EventView::removeCalendar(calendar);
+}
+
 Akonadi::Item::List TodoView::selectedIncidences() const
 {
     Akonadi::Item::List ret;
@@ -645,7 +702,7 @@ void TodoView::addTodo(const QString &summary, const Akonadi::Item &parentItem,
     // Use the same collection of the parent.
     if (parentItem.isValid()) {
         // Don't use parentColection() since it might be a virtual collection
-        collection = calendar()->collection(parentItem.storageCollectionId());
+        collection = Akonadi::EntityTreeModel::updatedCollection(model(), parentItem.storageCollectionId());
     }
 
     changer()->createIncidence(todo, collection, this);
@@ -692,7 +749,8 @@ void TodoView::contextMenu(QPoint pos)
                 // Action isn't RO, it can change the incidence, "Edit" for example.
                 const bool actionIsRw = mItemPopupMenuReadWriteEntries.contains(entry);
 
-                const bool incidenceIsRO = !calendar()->hasRight(item, Akonadi::Collection::CanChangeItem);
+                const auto collection = Akonadi::EntityTreeModel::updatedCollection(model(), item.storageCollectionId());
+                const bool incidenceIsRO = collection.rights() & Akonadi::Collection::CanChangeItem;
 
                 enable = hasItem && (!actionIsRw || !incidenceIsRO);
             }
@@ -708,9 +766,9 @@ void TodoView::contextMenu(QPoint pos)
     if (hasItem) {
         if (incidencePtr) {
             const bool hasRecId = incidencePtr->hasRecurrenceId();
-            if (calendar()) {
-                mMakeSubtodosIndependent->setEnabled(!hasRecId && !calendar()->childItems(incidencePtr->uid()).isEmpty());
-            }
+            const bool hasSubtodos = mView->model()->hasChildren(mView->indexAt(pos));
+
+            mMakeSubtodosIndependent->setEnabled(!hasRecId && hasSubtodos);
             mMakeTodoIndependent->setEnabled(!hasRecId && !incidencePtr->relatedTo().isEmpty());
         }
 
@@ -919,7 +977,8 @@ void TodoView::setNewDate(QDate date)
     KCalendarCore::Todo::Ptr todo = Akonadi::CalendarUtils::todo(todoItem);
     Q_ASSERT(todo);
 
-    if (calendar()->hasRight(todoItem, Akonadi::Collection::CanChangeItem)) {
+    const auto collection = Akonadi::EntityTreeModel::updatedCollection(model(), todoItem.storageCollectionId());
+    if (collection.rights() & Akonadi::Collection::CanChangeItem) {
         KCalendarCore::Todo::Ptr oldTodo(todo->clone());
         QDateTime dt(date.startOfDay());
 
@@ -949,7 +1008,8 @@ void TodoView::setStartDate(QDate date)
     KCalendarCore::Todo::Ptr todo = Akonadi::CalendarUtils::todo(todoItem);
     Q_ASSERT(todo);
 
-    if (calendar()->hasRight(todoItem, Akonadi::Collection::CanChangeItem)) {
+    const auto collection = Akonadi::EntityTreeModel::updatedCollection(model(), todoItem.storageCollectionId());
+    if (collection.rights() & Akonadi::Collection::CanChangeItem) {
         KCalendarCore::Todo::Ptr oldTodo(todo->clone());
         QDateTime dt(date.startOfDay());
 
@@ -979,7 +1039,8 @@ void TodoView::setNewPercentage(QAction *action)
     KCalendarCore::Todo::Ptr todo = Akonadi::CalendarUtils::todo(todoItem);
     Q_ASSERT(todo);
 
-    if (calendar()->hasRight(todoItem, Akonadi::Collection::CanChangeItem)) {
+    const auto collection = Akonadi::EntityTreeModel::updatedCollection(model(), todoItem.storageCollectionId());
+    if (collection.rights() & Akonadi::Collection::CanChangeItem) {
         KCalendarCore::Todo::Ptr oldTodo(todo->clone());
 
         int percentage = mPercentage.value(action);
@@ -1003,7 +1064,8 @@ void TodoView::setNewPriority(QAction *action)
     }
     const auto todoItem = selection[0].data(Akonadi::TodoModel::TodoRole).value<Akonadi::Item>();
     KCalendarCore::Todo::Ptr todo = Akonadi::CalendarUtils::todo(todoItem);
-    if (calendar()->hasRight(todoItem, Akonadi::Collection::CanChangeItem)) {
+    const auto collection = Akonadi::EntityTreeModel::updatedCollection(model(), todoItem.storageCollectionId());
+    if (collection.rights() & Akonadi::Collection::CanChangeItem) {
         KCalendarCore::Todo::Ptr oldTodo(todo->clone());
         todo->setPriority(mPriority[action]);
 
@@ -1021,7 +1083,8 @@ void TodoView::changedCategories(QAction *action)
     const auto todoItem = selection[0].data(Akonadi::TodoModel::TodoRole).value<Akonadi::Item>();
     KCalendarCore::Todo::Ptr todo = Akonadi::CalendarUtils::todo(todoItem);
     Q_ASSERT(todo);
-    if (calendar()->hasRight(todoItem, Akonadi::Collection::CanChangeItem)) {
+    const auto collection = Akonadi::EntityTreeModel::updatedCollection(model(), todoItem.storageCollectionId());
+    if (collection.rights() & Akonadi::Collection::CanChangeItem) {
         KCalendarCore::Todo::Ptr oldTodo(todo->clone());
 
         const QString cat = action->data().toString();
@@ -1079,7 +1142,7 @@ void TodoView::setFlatView(bool flatView, bool notifyOtherViews)
 
 void TodoView::onRowsInserted(const QModelIndex &parent, int start, int end)
 {
-    if (start != end || !calendar() || !calendar()->entityTreeModel()) {
+    if (start != end || !entityTreeModel()) {
         return;
     }
 
@@ -1096,7 +1159,7 @@ void TodoView::onRowsInserted(const QModelIndex &parent, int start, int end)
         return;
     }
 
-    const bool isPopulated = calendar()->entityTreeModel()->isCollectionPopulated(item.storageCollectionId());
+    const bool isPopulated = entityTreeModel()->isCollectionPopulated(item.storageCollectionId());
     if (!isPopulated) {
         return;
     }
@@ -1256,3 +1319,5 @@ void TodoView::createNote()
 
     Q_EMIT createNote(todoItem);
 }
+
+#include "todoview.moc"
